import datetime


from .author import ScirateAuthor
from .category import ScirateCategory
from .paper import SciratePaper
from .request import ScirateRequest


class ScirateClientException(Exception):
    def __init__(self, error_msg):
        self.error_msg = error_msg

    def __str__(self):
        return self.error_msg


class ScirateClient():
    base_url = "https://scirate.com"

    def __init__(self):
        """ """
        self.client_key = "KEY"

    @property
    def query_dict(self):
        return {"key": self.client_key}

    def request(self, *args, **kwargs):
        """Create a ScirateRequest object and make that request."""
        req = ScirateRequest(self, *args, **kwargs)
        return req.request()

    def author(self, first_name, last_name, category):
        """Get info about an author."""
        author_id = last_name + "_" + first_name[0] + "+in:" + category
        params = {"id": author_id, 
                  "first_name": first_name,
                  "last_name": last_name,
                  "category": category}

        resp = self.request("/search?q=au:", params, req_format="author")
        return ScirateAuthor(resp["author"], self)

    def paper(self, arxiv_id):
        """Get info about a paper."""
        params = {"id": arxiv_id}

        resp = self.request("/arxiv/", params, req_format="paper")
        return SciratePaper(resp["paper"], self)

    def category(self, category, date=datetime.datetime.today().strftime('%Y-%m-%d')):
        """Get arxiv listings for category."""
        search_id = category + "?date=" + date
        params = {"id": search_id,
                  "category": category,
                  "date": date}

        resp = self.request("/arxiv/", params, req_format="category")
        return ScirateCategory(resp["category"], self)



"""Scirate Requests class."""

import requests


from .parser import ScirateParser


class ScirateRequestException(Exception):
    """ScirateRequest Exception class."""
    def __init__(self, error_msg, url):
        self.error_msg = error_msg
        self.url = url

    def __str__(self):
        return self.url, ':', self.error_msg


class ScirateRequest():
    """ScirateRequest class."""
    def __init__(self, client, path, query_dict, req_format):
        """Initialize request object."""
        self.params = query_dict
        self.host = client.base_url
        self.path = path
        self.req_format = req_format

        self.parser = ScirateParser()

    def request(self):
        """Make web request to Scirate URL."""
        self.path += self.params["id"]
        resp = requests.get(self.host+self.path)

        if resp.status_code != 200:
            raise ScirateRequestException(resp.reason, self.path)
        elif self.req_format == "author":
            return self.parser.parse_author(resp, self.params)
        elif self.req_format == "paper":
            return self.parser.parse_paper(resp, self.params)
        elif self.req_format == "category":
            return self.parser.parse_category(resp, self.params)

        else:
            raise Exception("Invalid format.")



"""Scirate author class."""


class ScirateAuthor:
    """Scirate author information."""
    def __init__(self, author_dict, client):
        self._author_dict = author_dict
        self._client = client

    def __repr__(self):
        return self.name

    @property
    def gid(self):
        """Scirate id of author."""
        return self._author_dict["id"]

    @property
    def name(self):
        """Author name."""
        return self._author_dict["name"]

    @property
    def category(self):
        """Author publishing category."""
        return self._author_dict["category"]

    @property
    def papers(self):
        """Papers of the author."""
        return self._author_dict["papers"]

    @property
    def arxiv_ids(self):
        """arXiv ids of papers from author."""
        return self._author_dict["arxiv_ids"]

    @property
    def co_authors(self):
        """Co-authors of author for each paper."""
        return self._author_dict["co_authors"]



""" Scirate paper class."""
from . import author


class SciratePaper:
    """Scirate paper information."""
    def __init__(self, paper_dict, client):
        self._paper_dict = paper_dict
        self._client = client

    def __repr__(self):
        return self.title

    @property
    def arxiv_id(self):
        """Scirate id of paper."""
        return self._paper_dict["id"]

    @property
    def submitted_date(self):
        """Date of paper submission."""
        return self._paper_dict["submitted_date"]

    @property
    def published_date(self):
        """Date of paper publication."""
        return self._paper_dict["published_date"]

    @property
    def author_comments(self):
        """Author comments on paper."""
        return self._paper_dict["author_comments"]

    @property
    def title(self):
        """Title of paper."""
        return self._paper_dict["title"]

    @property
    def abstract(self):
        """Abstract of paper."""
        return self._paper_dict["abstract"]

    @property
    def scites(self):
        """Number of scites on paper."""
        return self._paper_dict["scites"]

    @property
    def scitors(self):
        """Users who scited paper."""
        return self._paper_dict["scitors"]

    @property
    def category(self):
        """Paper publishing category."""
        return self._paper_dict["category"]

    @property
    def authors(self):
        """Authors of paper."""
        return self._paper_dict["authors"]



import re
import requests
from bs4 import BeautifulSoup


from .util import find_substr_btw


class ScirateParserHelper():
    """Helper class for ScirateParser."""
    def __init__(self):
        self.base_url = "https://scirate.com"

    def author_pagination_url(self, page, params):
        """Create Scirate pagination URL."""
        return self.base_url+"/search?page="+str(page)+"&q=au%3A"+params["id"]+"+in%3A"+params["category"]

    def check_no_papers_found(self, url):
        """Returns true if pagination URL has no results."""
        resp = requests.get(url)
        soup = BeautifulSoup(resp.content, "lxml")
        paper_html = soup.findAll("div", attrs={"class": "title"})
        return paper_html == []

    def date_to_scirate_date_format(self, day, month, year):
        """Take in a day (integer), month (three-letter month code), and year
        (integer). Convert to Scirate date format YYYY-MM-DD."""
        months = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]
        return year + "-" + str(months.index(month) + 1) + "-" + day


class ScirateParserException(Exception):
    """Exception class for ScirateParser."""
    def __init__(self, error_msg):
        self.error_msg = error_msg

    def __str__(self):
        return self.error_msg


class ScirateParser():
    """Parser for Scirate class."""
    def __init__(self):
        """ """
        self.parser_helper = ScirateParserHelper()

    def parse_author(self, resp, params):
        """Parse author information from Scirate."""
        soup = BeautifulSoup(resp.content, "lxml")

        name = params["first_name"] + " " + params["last_name"]
        paper_html = soup.findAll("div", attrs={"class": "title"})
        author_html = soup.findAll("div", attrs={"class": "authors"})

        arxiv_ids = []
        papers = []
        co_authors = []

        page_count = 1
        page = self.parser_helper.author_pagination_url(page_count, params)

        while not self.parser_helper.check_no_papers_found(page):

            # Paper Titles and arXiv ids for Author:
            for div in paper_html:
                links = div.findAll("a")
                for a in links:
                    arxiv_id = a["href"].split("/arxiv/")
                    arxiv_ids.append(arxiv_id[1])
                    papers.append(a.contents[0])

            # Co-Authors for each Paper:
            for div in author_html:
                links = div.findAll("a")
                co_author = []
                for a in links:
                    co_author.append(a.contents[0])
                co_authors.append(co_author)

            page_count += 1
            page = self.parser_helper.author_pagination_url(page_count, params)

            resp = requests.get(page)
            soup = BeautifulSoup(resp.content, "lxml")
            paper_html = soup.findAll("div", attrs={"class": "title"})
            author_html = soup.findAll("div", attrs={"class": "authors"})

        author_dict = {"arxiv_ids": arxiv_ids,
                       "papers": papers,
                       "co_authors": co_authors,
                       "name": name,
                       "id": params["id"],
                       "category": params["category"]}

        data_dict = {"ScirateResponse": {"author": author_dict}}
        return data_dict["ScirateResponse"]

    def parse_paper(self, resp, params):
        """Parse paper information from Scirate."""
        soup = BeautifulSoup(resp.content, "lxml")

        title_html = soup.findAll("h1", attrs={"class": "title"})
        abstract_html = soup.findAll("div", attrs={"class": "abstract"})
        scites_html = soup.findAll("button", attrs={"class": "btn btn-default count"})
        scitors_html = soup.findAll("div", attrs={"class": "scites"})
        author_html = soup.findAll("ul", attrs={"class": "authors"})
        date_html = soup.findAll("div", attrs={"class": "dates"})

        # Paper Title:
        title = title_html[0].text

        # Paper Abstract:
        abstract = abstract_html[0].text

        # Paper Scites:
        scites = int(scites_html[0].text)

        date = date_html[0].text
        submitted_date = find_substr_btw(date, "Submitted", "to").rstrip().lstrip()
        published_date = find_substr_btw(date, "Published", "").rstrip().lstrip()

        # Submission Date:
        submitted_date_split = submitted_date.split()
        submitted_day = submitted_date_split[0]
        submitted_month = submitted_date_split[1]
        submitted_year = submitted_date_split[2]
        submitted_date = self.parser_helper.date_to_scirate_date_format(submitted_day, submitted_month, submitted_year)

        # Publication Date:
        published_date_split = published_date.split()
        published_day = published_date_split[0]
        published_month = published_date_split[1]
        published_year = published_date_split[2]
        published_date = self.parser_helper.date_to_scirate_date_format(published_day, published_month, published_year)

        # Author Comments from arXiv:
        author_comments = soup.body.findAll(text=re.compile("Author comments:"))
        author_comments = author_comments[0].split(":")[1].rstrip().lstrip()

        # arXiv Category:
        category = ""
        for div in date_html:
            links = div.findAll("a")
            for a in links:
                category = a.contents[0]

        # Authors on Paper:
        authors = []
        for div in author_html:
            links = div.findAll("a")
            for a in links:
                authors.append(a.contents[0])

        # Scirate Users who Scited Paper:
        scitors = []
        for div in scitors_html:
            links = div.findAll("a")
            for a in links:
                scitors.append(a.contents[0])

        paper_dict = {"id": params["id"],
                      "title": title,
                      "abstract": abstract,
                      "category": category,
                      "submitted_date": submitted_date,
                      "published_date": published_date,
                      "scites": scites,
                      "scitors": scitors,
                      "author_comments": author_comments,
                      "authors": authors}

        data_dict = {"ScirateResponse": {"paper": paper_dict}}
        return data_dict["ScirateResponse"]

    def parse_category(self, resp, params):
        soup = BeautifulSoup(resp.content, "lxml")

        paper_html = soup.findAll("div", attrs={"class": "title"})
        author_html = soup.findAll("div", attrs={"class": "authors"})
        scites_html = soup.findAll("button", attrs={"class": "btn btn-default count"})

        arxiv_ids = []
        co_authors = []
        papers = []
        scites = []

        # Paper Scites:
        for scite in scites_html:
            scites.append(int(scite.text))

        # Paper Titles and arXiv ids for Author:
        for div in paper_html:
            links = div.findAll("a")
            for a in links:
                arxiv_id = a["href"].split("/arxiv/")
                arxiv_ids.append(arxiv_id[1])
                papers.append(a.contents[0])

        # Co-Authors for each Paper:
        for div in author_html:
            links = div.findAll("a")
            co_author = []
            for a in links:
                co_author.append(a.contents[0].strip(","))
            co_authors.append(co_author)

        category_dict = {"id": params["id"],
                         "category": params["category"],
                         "date": params["date"],
                         "papers": papers,
                         "arxiv_ids": arxiv_ids,
                         "co_authors": co_authors,
                         "scites": scites}

        data_dict = {"ScirateResponse": {"category": category_dict}}
        return data_dict["ScirateResponse"]



"""Scirate category class."""


class ScirateCategory:
    """Scirate category information."""
    def __init__(self, category_dict, client):
        self._category_dict = category_dict
        self._client = client

    def __repr__(self):
        return self.category

    @property
    def gid(self):
        """Scirate id of category query."""
        return self._category_dict["id"]

    @property
    def category(self):
        """Publishing category."""
        return self._category_dict["category"]

    @property
    def papers(self):
        """Papers of category."""
        return self._category_dict["papers"]

    @property
    def date(self):
        """Date of papers in category."""
        return self._category_dict["date"]

    @property
    def arxiv_ids(self):
        """arXiv ids of papers in category."""
        return self._category_dict["arxiv_ids"]

    @property
    def co_authors(self):
        """Co-authors for each paper in category."""
        return self._category_dict["co_authors"]

    @property
    def scites(self):
        """Number of scites for each paper in category."""
        return self._category_dict["scites"]



from setuptools import setup

setup(
    name="scirate",
    description="Python wrapper for extracting content from Scirate.",
    long_description=open("README.rst").read(),
    url="https://github.com/vprusso/scirate",

    author="Vincent Russo",
    author_email="vincentrusso1@gmail.com",

    packages=["scirate"],
    version="0.0.2",
    install_requires=['nose', 'bs4', 'lxml', 'requests'],

    license="MIT",

    keywords=["scirate", "scirate API", "scirate python"],

    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5",
    ],

    project_urls={
        "Homepage": "http://vprusso.github.io/",
    }
)



"""General purpose utility functions."""
import re


def find_substr_btw(s, start, end):
    """Find substring in string s between start and end."""
    return re.search("%s(.*)%s" % (start, end), s).group(1)
